import time
import Adafruit_GPIO as GPIO
import ft
import ctypes
import PeriodicThread
import collections

#REGISTERS
REG_FIFO        		     =	0x00
REG_OP_MODE        		     =	0x01
#REG_BITRATE_MSB    	     =	0x02
#REG_BITRATE_LSB    	     =	0x03
#REG_FDEV_MSB   		     =	0x04
#REG_FDEV_LSB    		     =	0x05
REG_FRF_MSB    			     =	0x06
REG_FRF_MID    			     =	0x07
REG_FRF_LSB    			     =	0x08
REG_PA_CONFIG    		     =	0x09
REG_PA_RAMP    			     =	0x0A
REG_OCP    				     =	0x0B
REG_LNA    				     =	0x0C
#REG_RX_CONFIG    		     =	0x0D
REG_FIFO_ADDR_PTR  		     =	0x0D
#REG_RSSI_CONFIG   		     =	0x0E
REG_FIFO_TX_BASE_ADDR 	     =	0x0E
#REG_RSSI_COLLISION    	     =	0x0F
REG_FIFO_RX_BASE_ADDR        =	0x0F
#REG_RSSI_THRESH    	     =	0x10
REG_FIFO_RX_CURRENT_ADDR     =  0x10
#REG_RSSI_VALUE_FSK	         =	0x11
REG_IRQ_FLAGS_MASK    	     =	0x11
#REG_RX_BW		    	     =	0x12
REG_IRQ_FLAGS	    	     =	0x12
#REG_AFC_BW		    	     =	0x13
REG_RX_NB_BYTES	    	     =	0x13
#REG_OOK_PEAK	    	     =	0x14
REG_RX_HEADER_CNT_VALUE_MSB  = 	0x14
#REG_OOK_FIX	    		 =	0x15
REG_RX_HEADER_CNT_VALUE_LSB  =	0x15
#REG_OOK_AVG	 			 =	0x16
REG_RX_PACKET_CNT_VALUE_MSB  =	0x16
REG_RX_PACKET_CNT_VALUE_LSB  =	0x17
REG_MODEM_STAT	  			 =	0x18
REG_PKT_SNR_VALUE	  		 =	0x19
#REG_AFC_FEI	  			 =	0x1A
REG_PKT_RSSI_VALUE	  		 =	0x1A
#REG_AFC_MSB	  			 =	0x1B
REG_RSSI_VALUE_LORA	  		 =	0x1B
#REG_AFC_LSB	  			 =	0x1C
REG_HOP_CHANNEL	  			 =	0x1C
#REG_FEI_MSB	  			 =	0x1D
REG_MODEM_CONFIG1	 		 =	0x1D
#REG_FEI_LSB	  			 =	0x1E
REG_MODEM_CONFIG2	  		 =	0x1E
#REG_PREAMBLE_DETECT  		 =	0x1F
REG_SYMB_TIMEOUT_LSB  		 =	0x1F
#REG_RX_TIMEOUT1	  		 =	0x20
REG_PREAMBLE_MSB_LORA  		 =	0x20
REG_PREAMBLE_LSB_LORA  		 =	0x21
#REG_RX_TIMEOUT2	  		 =	0x21
#REG_RX_TIMEOUT3	 		 =	0x22
REG_PAYLOAD_LENGTH_LORA		 =	0x22
#REG_RX_DELAY	 			 =	0x23
REG_MAX_PAYLOAD_LENGTH 		 =	0x23
#REG_OSC		 			 =	0x24
REG_HOP_PERIOD	  			 =	0x24
#REG_PREAMBLE_MSB_FSK 		 =	0x25
REG_FIFO_RX_BYTE_ADDR 		 =	0x25
#REG_PREAMBLE_LSB_FSK 		 =	0x26
#REG_SYNC_CONFIG	  		 =	0x27
#REG_SYNC_VALUE1	 		 =	0x28
REG_FEI_MSB                  =  0X28
REG_FEI_MID					 =	0X29
#REG_SYNC_VALUE2	  		 =	0x29
#REG_SYNC_VALUE3	  		 =	0x2A
REG_FEI_LSB					 =	0X2A
#REG_SYNC_VALUE4	  		 =	0x2B
REG_SYNC_VALUE5	  			 =	0x2C
#REG_SYNC_VALUE6	  		 =	0x2D
#REG_SYNC_VALUE7	  		 =	0x2E
#REG_SYNC_VALUE8	  		 =	0x2F
#REG_PACKET_CONFIG1	  		 =	0x30
#REG_PACKET_CONFIG2	  		 =	0x31
REG_DETECT_OPTIMIZE          =  0x31
#REG_PAYLOAD_LENGTH_FSK		 =	0x32
#REG_NODE_ADRS	  			 =	0x33
REG_INVERT_IQ				 =	0X33
#REG_BROADCAST_ADRS	 		 = 	0x34
#REG_FIFO_THRESH	  		 =	0x35
#REG_SEQ_CONFIG1	  		 =	0x36
#REG_SEQ_CONFIG2	  		 =	0x37
REG_DETECTION_THRESHOLD      =  0x37
#REG_TIMER_RESOL	  		 =	0x38
#REG_TIMER1_COEF	  		 =	0x39
REG_SYNC_WORD				 =	0X39
#REG_TIMER2_COEF	  		 =	0x3A
#REG_IMAGE_CAL	  			 =	0x3B
#REG_TEMP		  			 =	0x3C
#REG_LOW_BAT	  			 =	0x3D
#REG_IRQ_FLAGS1	  			 =	0x3E
#REG_IRQ_FLAGS2	  			 =	0x3F
REG_DIO_MAPPING1	  		 =	0x40
REG_DIO_MAPPING2	  		 =	0x41
REG_VERSION	  				 =	0x42
REG_AGC_REF	  				 =	0x43
REG_AGC_THRESH1	  			 =	0x44
REG_AGC_THRESH2	  			 =	0x45
REG_AGC_THRESH3	  			 =	0x46
REG_PLL_HOP	  				 =	0x4B
REG_TCXO		  			 =	0x58
REG_PA_DAC		  			 =	0x5A
REG_PLL		  				 =	0x5C
REG_PLL_LOW_PN	  			 =	0x5E
REG_FORMER_TEMP	  			 =	0x6C
REG_BIT_RATE_FRAC	  		 =	0x70

#FREQUENCY CHANNELS:
CH_10_868 = 0xD84CCC # channel 10, central freq = 865.20MHz
CH_11_868 = 0xD86000 # channel 11, central freq = 865.50MHz
CH_12_868 = 0xD87333 # channel 12, central freq = 865.80MHz
CH_13_868 = 0xD88666 # channel 13, central freq = 866.10MHz
CH_14_868 = 0xD89999 # channel 14, central freq = 866.40MHz
CH_15_868 = 0xD8ACCC # channel 15, central freq = 866.70MHz
CH_16_868 = 0xD8C000 # channel 16, central freq = 867.00MHz
CH_17_868 = 0xD90000 # channel 16, central freq = 868.00MHz
CH_00_900 = 0xE1C51E # channel 00, central freq = 903.08MHz
CH_01_900 = 0xE24F5C # channel 01, central freq = 905.24MHz
CH_02_900 = 0xE2D999 # channel 02, central freq = 907.40MHz
CH_03_900 = 0xE363D7 # channel 03, central freq = 909.56MHz
CH_04_900 = 0xE3EE14 # channel 04, central freq = 911.72MHz
CH_05_900 = 0xE47851 # channel 05, central freq = 913.88MHz
CH_06_900 = 0xE5028F # channel 06, central freq = 916.04MHz
CH_07_900 = 0xE58CCC # channel 07, central freq = 918.20MHz
CH_08_900 = 0xE6170A # channel 08, central freq = 920.36MHz
CH_09_900 = 0xE6A147 # channel 09, central freq = 922.52MHz
CH_10_900 = 0xE72B85 # channel 10, central freq = 924.68MHz
CH_11_900 = 0xE7B5C2 # channel 11, central freq = 926.84MHz
CH_12_900 = 0xE4C000 # default channel 915MHz, the module is configured with it


#LORA BANDWIDTH:
BW_125 = 0x00
BW_250 = 0x01
BW_500 = 0x02


#LORA CODING RATE:
CR_5 = 0x01
CR_6 = 0x02
CR_7 = 0x03
CR_8 = 0x04

#LORA SPREADING FACTOR:
SF_6 = 0x06
SF_7 = 0x07
SF_8 = 0x08
SF_9 = 0x09
SF_10 = 0x0A
SF_11 = 0x0B
SF_12 = 0x0C

#LORA MODES:
LORA_SLEEP_MODE = 0x80
LORA_STANDBY_MODE = 0x81
LORA_TX_MODE = 0x83
LORA_RX_MODE = 0x85
LORA_STANDBY_FSK_REGS_MODE = 0xC1

#FSK MODES:
FSK_SLEEP_MODE = 0x00
#FSK_STANDBY_MODE = 0x01
#FSK_TX_MODE = 0x03
#FSK_RX_MODE = 0x05

#OTHER CONSTANTS:
HEADER_ON = 0
HEADER_OFF = 1
CRC_ON = 1
CRC_OFF = 0
LORA = 1
FSK = 0
BROADCAST_0 = 0x00
MAX_LENGTH = 255
MAX_PAYLOAD = 251
ACK_LENGTH = 5
OFFSET_PAYLOADLENGTH = 5 
OFFSET_RSSI = 137
NOISE_FIGURE = 6.0
NOISE_ABSOLUTE_ZERO = 174.0
MAX_TIMEOUT = 8000
MAX_WAIT = 12000
MAX_RETRIES = 5
CORRECT_PACKET = 0
INCORRECT_PACKET = 1

# Operation mode only for Lora
class OpModeBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("Mode",	ctypes.c_uint8, 3),
		("Unused",	ctypes.c_uint8, 3),
		("AccesSharedReg",	ctypes.c_uint8, 1),
		("LongRangeMode",	ctypes.c_uint8, 1),
	]


class RegOpMode(ctypes.Union):
	_fields_ = [
		("bits",	OpModeBits),
		("octet",	ctypes.c_uint8)
	]

class RegModemConfig1Bits(ctypes.LittleEndianStructure):
	_fields_ = [
		("LowDataRateOptimize",	ctypes.c_uint8,1),
		("RxPayloadCrcOn",	ctypes.c_uint8,1),
		("ImplicitHeaderModeOn",	ctypes.c_uint8,1),
		("CodingRate",	ctypes.c_uint8,3),
		("Bw",	ctypes.c_uint8,2),
	]					

class RegModemConfig1(ctypes.Union):
	_fields_ = [
		("bits", RegModemConfig1Bits),
		("octet", ctypes.c_uint8)
	]


class RegModemConfig2Bits(ctypes.LittleEndianStructure):
	_fields_ = [
		("SymbTimeout(9:8)",	ctypes.c_uint8,2),
		("AgcAutoOn",	ctypes.c_uint8,1),
		("TxContinuousMode",	ctypes.c_uint8,1),
		("SpreadingFactor",	ctypes.c_uint8,4),
	]

class RegModemConfig2(ctypes.Union):
	_fields_ = [
		("bits", RegModemConfig2Bits),
		("octet", ctypes.c_uint8)
	]

class adressBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("others",	ctypes.c_uint8, 7),
		("wnr",	ctypes.c_uint8, 1),		# wnr=0 for read and wnr=1 for write
	]

class adress1(ctypes.Union):
	_fields_ = [
		("bits",	adressBits),
		("octet",	ctypes.c_uint8)
	]

class preambleBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("LSB",	ctypes.c_uint16, 8),
		("MSB",	ctypes.c_uint16, 8),		
	]

class preamble(ctypes.Union):
	_fields_ = [
		("bits",	preambleBits),
		("octet",	ctypes.c_uint16)
    ]  

class frfBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("LSB",	ctypes.c_uint8, 8),
		("MIB",	ctypes.c_uint8, 8),
		("MSB",	ctypes.c_uint8, 8),
		
	]

class frf(ctypes.Union):
	_fields_ = [
		("bits",	frfBits),
		("octet",	ctypes.c_uint)
	]

class powerBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("OutputPower",	ctypes.c_uint8, 4),
		("Unused",	ctypes.c_uint8, 3),
		("PaSelect",	ctypes.c_uint8, 1),

	]
class power(ctypes.Union):
	_fields_ = [
		("bits",	powerBits),
		("octet",	ctypes.c_uint8)
	]

class flagsBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("CadDetected",	ctypes.c_uint8, 1),
		("FhssChangeChannel",	ctypes.c_uint8, 1),
		("CadDone",	ctypes.c_uint8, 1),
		("TxDone",	ctypes.c_uint8, 1),
		("ValidHeader",	ctypes.c_uint8, 1),
		("PayloadCrcError",	ctypes.c_uint8, 1),
		("RxDone",	ctypes.c_uint8, 1),
		("RxTimeout",	ctypes.c_uint8, 1),
		
	]
class flags(ctypes.Union):
	_fields_ = [
		("bits", flagsBits),
		("octet", ctypes.c_uint8)
	]


class HopChannelBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("FhssPresentChannel",	ctypes.c_uint8, 6),
		("CrcOnPayload",	ctypes.c_uint8, 1),
		("PllTimeout",	ctypes.c_uint8, 1),
		
	]

class HopChannel(ctypes.Union):
	_fields_ = [
		("bits",	HopChannelBits),
		("octet",	ctypes.c_uint8)
	]

class ValidpacketsRXBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("ValidPacketCntLsb",	ctypes.c_uint8, 8),
		("ValidPacketCntMsb",	ctypes.c_uint8, 8),
	
	]

class ValidpacketsRX(ctypes.Union):
	_fields_ = [
		("bits",	ValidpacketsRXBits),
		("octet",	ctypes.c_uint16)
	]

class ValidheadersRXBits(ctypes.LittleEndianStructure):
	_fields_ = [
		("ValidHeaderCntLsb",	ctypes.c_uint8, 8),
		("ValidHeaderCntMsb",	ctypes.c_uint8, 8),
		
	]

class ValidheadersRX(ctypes.Union):
	_fields_ = [
		("bits",	ValidheadersRXBits),
		("octet",	ctypes.c_uint16)
	]



class SX1272:
	def __init__(self):
		#Initialize class variables
		self.bandwidth = BW_125
		self.codingRate = CR_5
		self.spreadingFactor = SF_7
		self.channel = CH_12_900
		self.header = HEADER_ON
		self.CRC = CRC_OFF
		self.modem = FSK
		self.power = 15
		self.packetNumber = 0
		#self.reception = CORRECT_PACKET
		#self.retries = 0
		#self.maxRetries = 3
		#self.packet_sent_retry = 0
		self.queuerx=collections.deque()
		self.threadrx=PeriodicThread.PeriodicThread(self.getonePacket,0.01)
		self.rx=False


	def ON(self):
		# Temporarily disable FTDI serial drivers.
		ft.use_FT232H()
 		# Find the first FT232H device.
		self.ft232h = ft.FT232H()
		# Make pin D4 a digital output, conected to pin 6 on Lora module.
		self.ft232h.setup(4, GPIO.OUT)


		# reset pulse for LoRa module initialization
		self.ft232h.output(4, GPIO.HIGH) 
		time.sleep(0.1)
		self.ft232h.output(4, GPIO.LOW)
		time.sleep(0.1)
		self.bandwidth = BW_125
		self.codingRate = CR_5
		self.spreadingFactor = SF_7
		self.channel = CH_12_900
		self.header = HEADER_ON
		self.CRC = CRC_OFF
		self.modem = FSK
		self.power = 15
		self.packetNumber = 0
		#self.reception = CORRECT_PACKET
		#self.retries = 0
		#self.maxRetries = 3
		#self.packet_sent_retry = 0
		# Create a SPI interface from the FT232H using pin 3 (D3) as chip select.
		# Use a clock speed of 3mhz, SPI mode 0, and most significant bit first.
		self.spi = ft.SPI(self.ft232h, cs=3, max_speed_hz=3000000, mode=0, bitorder=ft.MSBFIRST)
		self.ko=0	#Variable para controlar si configuracion es correcta


	def readRegister(self,adress):
		a=adress1()
		a.octet=adress
		a.bits.wnr=0
		self.spi.writeR([a.octet]) # Nose si usar funcion spi.transfer()
		response=self.spi.read(1)
		return response[0]

	def writeRegister(self,adress,data):
		a=adress1()
		a.octet=adress
		a.bits.wnr=1
		self.spi.write([a.octet,data])

	def clearFlags(self):
		if(self.modem==LORA):
			st0 = self.readRegister(REG_OP_MODE)		#Save the previous status
			self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	# Stdby mode to write in registers
			self.writeRegister(REG_IRQ_FLAGS, 0xFF)	# LoRa mode flags register
			self.writeRegister(REG_OP_MODE, st0)		# Getting back to previous status
		else:
			print('Not in LORA mode')

	def setLORA(self):
		self.writeRegister(REG_OP_MODE, FSK_SLEEP_MODE)    # Sleep mode (mandatory to set LoRa mode)
		self.writeRegister(REG_OP_MODE, LORA_SLEEP_MODE)    # LoRa sleep mode
		self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	# LoRa standby mode

		self.writeRegister(REG_MAX_PAYLOAD_LENGTH,MAX_LENGTH)

		time.sleep(0.1)

		st0 = self.readRegister(REG_OP_MODE)	# Reading config mode
		
		if(st0==LORA_STANDBY_MODE):
			self.modem=LORA
			#print('## LoRa set with success')
		else:
			print('** There has been an error while setting LoRa **')
			self.ko=1

	def getMode(self):
		if(self.modem==LORA):
			value = self.readRegister(REG_MODEM_CONFIG1)
			a=RegModemConfig1()
			a.octet=value
			self.bandwidth=a.bits.Bw
			print 'bandwidth =' ,
			print hex(self.bandwidth)
			self.codingRate=a.bits.CodingRate
			print 'codingRate =' ,
			print hex(self.codingRate)
			value = self.readRegister(REG_MODEM_CONFIG2)
			a=RegModemConfig2()
			a.octet=value
			self.spreadingFactor=a.bits.SpreadingFactor
			print 'SpreadingFactor=' ,
			print hex(self.spreadingFactor)
		else:
			print('Not in LORA mode')

	def setMode(self,mode):
		if(self.modem==LORA):
			st0 = self.readRegister(REG_OP_MODE)		#Save the previous status

			self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa standby mode
			c1=RegModemConfig1()
			c2=RegModemConfig2()
			value1=self.readRegister(REG_MODEM_CONFIG1)
			value2=self.readRegister(REG_MODEM_CONFIG2)
			c1.octet=value1
			c2.octet=value2

			if(mode==1):	#mode 1 (better reach, medium time on air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_12
				c1.bits.Bw=BW_125
				

			elif(mode==2):	#mode 2 (medium reach, less time on air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_12
				c1.bits.Bw=BW_250

			elif(mode==3):	#mode 3 (worst reach, less time on air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_10
				c1.bits.Bw=BW_125

			elif(mode==4):	#mode 4 (better reach, low time on air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_12
				c1.bits.Bw=BW_500

			elif(mode==5):	#mode 5 (better reach, medium time on air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_10
				c1.bits.Bw=BW_250

			elif(mode==6):	#mode 6 (better reach, worst time-on-air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_11
				c1.bits.Bw=BW_500

			elif(mode==7):	#mode 7 (medium-high reach, medium-low time-on-air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_9
				c1.bits.Bw=BW_250

			elif(mode==8):	#mode 8 (medium reach, medium time-on-air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_9
				c1.bits.Bw=BW_500

			elif(mode==9):	#mode 9 (medium-low reach, medium-high time-on-air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_8
				c1.bits.Bw=BW_500

			elif(mode==10):	#mode 10 (worst reach, less time_on_air)
				c1.bits.CodingRate=CR_5
				c2.bits.SpreadingFactor=SF_7
				c1.bits.Bw=BW_500
			else:
				print("** The indicated mode doesn't exist, ")
				print("please select from 1 to 10 **")
				self.ko=1

			self.bandwidth = c1.bits.Bw
			self.codingRate = c1.bits.CodingRate
			self.spreadingFactor = c2.bits.SpreadingFactor
			self.writeRegister(REG_MODEM_CONFIG1,c1.octet)
			self.writeRegister(REG_MODEM_CONFIG2,c2.octet)
			self.writeRegister(REG_OP_MODE, st0)
		else:
			print('Not in LORA mode')

	def setCodingRate(self,codingrate):
		if(self.modem==LORA):
			st0 = self.readRegister(REG_OP_MODE)		#Save the previous status
			self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa standby mode
			c=RegModemConfig1()
			value=self.readRegister(REG_MODEM_CONFIG1)
			c.octet=value
			if (codingrate==5):
				c.bits.CodingRate=CR_5
			elif (codingrate==6):
				c.bits.CodingRate=CR_6
			elif (codingrate==7):
				c.bits.CodingRate=CR_7
			elif (codingrate==8):
				c.bits.CodingRate=CR_8
			else:
				print('Invalid CodingRate')
				self.ko=1
			self.codingRate = c.bits.CodingRate
			self.writeRegister(REG_MODEM_CONFIG1,c.octet)
			self.writeRegister(REG_OP_MODE, st0)
		else:
			print('Not in LORA mode')

	
	def setSpreadingFactor(self,spreading):
		if(self.modem==LORA):
			st0 = self.readRegister(REG_OP_MODE)		#Save the previous status
			self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa standby mode
			c=RegModemConfig2()
			value=self.readRegister(REG_MODEM_CONFIG2)
			c.octet=value
			if (spreading==6):
				c.bits.SpreadingFactor=SF_6
			elif (spreading==7):
				c.bits.SpreadingFactor=SF_7
			elif (spreading==8):
				c.bits.SpreadingFactor=SF_8
			elif (spreading==9):
				c.bits.SpreadingFactor=SF_9
			elif (spreading==10):
				c.bits.SpreadingFactor=SF_10
			elif (spreading==11):
				c.bits.SpreadingFactor=SF_11
			elif (spreading==12):
				c.bits.SpreadingFactor=SF_12
			else:
				print('Invalid SpreadingFactor')
				self.ko=1
			self.spreadingFactor = c.bits.SpreadingFactor
			self.writeRegister(REG_MODEM_CONFIG2,c.octet)
			self.writeRegister(REG_OP_MODE, st0)
		else:
			print('Not in LORA mode')	

	def setBandwidth(self,bandwidth):
		if(self.modem==LORA):
			st0 = self.readRegister(REG_OP_MODE)		#Save the previous status
			self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa standby mode
			c=RegModemConfig1()
			value=self.readRegister(REG_MODEM_CONFIG1)
			c.octet=value
			if (bandwidth==125):
				c.bits.Bw=BW_125
			elif (bandwidth==250):
				c.bits.Bw=BW_250
			elif (bandwidth==500):
				c.bits.Bw=BW_500
			else:
				print('Invalid Bandwidth')
				self.ko=1
			self.bandwidth = c.bits.Bw
			self.writeRegister(REG_MODEM_CONFIG1,c.octet)
			self.writeRegister(REG_OP_MODE, st0)
		else:
			print('Not in LORA mode')
	
	def setLowDataRateOptimize(self):
		if(self.modem==LORA):
			st0 = self.readRegister(REG_OP_MODE)		#Save the previous status
			self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa standby mode
			c=RegModemConfig1()
			value=self.readRegister(REG_MODEM_CONFIG1)
			c.octet=value
			if (self.spreadingFactor==SF_11 and self.bandwidth==BW_125):
				c.bits.LowDataRateOptimize=1
			elif (self.spreadingFactor==SF_12 and self.bandwidth==BW_125):
				c.bits.LowDataRateOptimize=1
			else:
				c.bits.LowDataRateOptimize=0
			self.writeRegister(REG_MODEM_CONFIG1,c.octet)
			self.writeRegister(REG_OP_MODE, st0)
		else:
			print('Not in LORA mode')



	def getHeader(self):
		c1=RegModemConfig1()
		value1=self.readRegister(REG_MODEM_CONFIG1)
		c1.octet=value1
		if(self.modem==FSK):
			print("## Notice that FSK mode packets hasn't header ##")
		else:
			if (c1.bits.ImplicitHeaderModeOn==0):
				self.header=HEADER_ON
				print("## Header is in explicit header mode ##")
			else:
				self.header=HEADER_OFF
				print("## Header is in implicit header mode ##")


	def setHeaderON(self): #Explicit mode
		if(self.modem==FSK):
			print("## FSK mode packets hasn't header ##")
			self.ko=1
		else:
			if(self.spreadingFactor==SF_6):
				print("## Mandatory implicit header mode with spreading factor = 6 ##")
				self.ko=1
			else:
				c1=RegModemConfig1()
				value1=self.readRegister(REG_MODEM_CONFIG1)
				c1.octet=value1
				c1.bits.ImplicitHeaderModeOn=0
				self.writeRegister(REG_MODEM_CONFIG1,c1.octet)
				self.header=HEADER_ON
				#print("## Header has been activated ##")

	def setHeaderOFF(self): #Implicit mode
		if(self.modem==FSK):
			print("## FSK mode packets hasn't header ##")
			self.ko=1
		else:
			c1=RegModemConfig1()
			value1=self.readRegister(REG_MODEM_CONFIG1)
			c1.octet=value1
			c1.bits.ImplicitHeaderModeOn=1
			self.writeRegister(REG_MODEM_CONFIG1,c1.octet)
			self.header=HEADER_OFF
			#print("## Header has been desactivated ##")

	def getCRC(self):
		if(self.modem==LORA):
			c1=RegModemConfig1()
			value1=self.readRegister(REG_MODEM_CONFIG1)
			c1.octet=value1
			if (c1.bits.RxPayloadCrcOn==CRC_ON):
				self.CRC=CRC_ON
				print("## CRC is activated ##")
			else:
				self.CRC=CRC_OFF
				print("## CRC is desactivated ##")
		else:
			print("Not in LORA mode")

	def setCRC_ON(self):
		if(self.modem==LORA):
			c1=RegModemConfig1()
			value1=self.readRegister(REG_MODEM_CONFIG1)
			c1.octet=value1
			c1.bits.RxPayloadCrcOn=1
			self.CRC=CRC_ON
			self.writeRegister(REG_MODEM_CONFIG1,c1.octet)
			#print("## CRC has been activated ##")

		else:
			print("Not in LORA mode")
			
	def setCRC_OFF(self):
		if(self.modem==LORA):
			c1=RegModemConfig1()
			value1=self.readRegister(REG_MODEM_CONFIG1)
			c1.octet=value1
			c1.bits.RxPayloadCrcOn=0
			self.CRC=CRC_OFF
			self.writeRegister(REG_MODEM_CONFIG1,c1.octet)
			#print("## CRC has been desactivated ##")

		else:
			print("Not in LORA mode")
	 		

	def getPreambleLength(self):
		if(self.modem==LORA):
			pre=preamble()
			pre.bits.LSB=self.readRegister(REG_PREAMBLE_LSB_LORA)
			pre.bits.MSB=self.readRegister(REG_PREAMBLE_MSB_LORA)
			self.preamblelength=pre.octet
			print "## Preamble length configured is " ,
			print(hex(self.preamblelength))
		else:
			print("Not in LORA mode")

	def setPreambleLength(self,l):
		if(self.modem==LORA):
			if (l>=6 and l<=65535):
				st0 = self.readRegister(REG_OP_MODE)
				pre=preamble()
				pre.octet=l
				self.preamblelenght=l
				self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE) #Set Standby mode to write in registers
				self.writeRegister(REG_PREAMBLE_MSB_LORA, pre.bits.MSB)
				self.writeRegister(REG_PREAMBLE_LSB_LORA, pre.bits.LSB)
				self.writeRegister(REG_OP_MODE, st0)		#Getting back to previous status
			else:
				print('l out of range 6<=l<=65535')
				self.ko=1
		else:
			print("Not in LORA mode")

	def getChannel(self):
		if(self.modem==LORA):
			fr=frf()
			fr.bits.MSB=self.readRegister(REG_FRF_MSB)
			fr.bits.MIB=self.readRegister(REG_FRF_MID)
			fr.bits.LSB=self.readRegister(REG_FRF_LSB)
			self.channel=fr.octet
			freq=float(32*(fr.octet))/(2**19)
			print 'channel:',
			print(freq)
		else:
			print("Not in LORA mode")

	def setChannel(self,channel): #channel in Mhz
		if(self.modem==LORA):
			if(channel>0 and channel<1024):
				st0 = self.readRegister(REG_OP_MODE)	#Save the previous status
				Frf=int((channel*(2**19))/32)
				a=frf()
				a.octet=Frf
				self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa Stdby mode in order to write in registers
				self.writeRegister(REG_FRF_MSB, a.bits.MSB )
				self.writeRegister(REG_FRF_MID, a.bits.MIB)
				self.writeRegister(REG_FRF_LSB, a.bits.LSB)
				self.writeRegister(REG_OP_MODE, st0)
				self.channel=Frf
			else:
				print('Channel out of range')
				self.ko=1
		else:
			print("Not in LORA mode")

	def getPower(self):
		if(self.modem==LORA):
			pow=power()
			pow.octet=self.readRegister(REG_PA_CONFIG)
			print "## Output power is ",
			if pow.bits.PaSelect==1:
				print 2+pow.bits.OutputPower,
			else:
				print -1+pow.bits.OutputPower,
			print 'dBm'
		else:
			print("Not in LORA mode")

	def setPower(self,p):	# p= M(max),L(low) or H(high)
		if(self.modem==LORA):
			st0 = self.readRegister(REG_OP_MODE)	   #Save the previous status
			self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa Stdby mode to write in registers
			pow=power()
			pow.octet=self.readRegister(REG_PA_CONFIG)
			if (p=='M'):
				self.power=0x0F
			elif (p=='L'):
				self.power=0x00
			elif (p=='H'):
				self.power=0x07
			else:
				print'Not M(max),L(low) or H(high) selected'
			pow.bits.OutputPower=self.power
			self.writeRegister(REG_PA_CONFIG,pow.octet)
			self.writeRegister(REG_OP_MODE, st0)		# Getting back to previous status
		else:
			print("Not in LORA mode")

	def setPowerNum(self,pow):	#pow = Pout (-1 to 17 dBm), RFIO enabled
		if(self.modem==LORA):
			if (pow>=-1 and pow<=14):
				st0 = self.readRegister(REG_OP_MODE)	   #Save the previous status
				self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa Stdby mode to write in registers
				self.power=pow+1
				p=power()
				p.octet=self.readRegister(REG_PA_CONFIG)
				p.bits.OutputPower=self.power
				self.writeRegister(REG_PA_CONFIG,p.octet)
				self.writeRegister(REG_OP_MODE, st0)		# Getting back to previous status
			
			elif (pow>14 and pow<=17): #PA_BOOST enabled
				st0 = self.readRegister(REG_OP_MODE)	   #Save the previous status
				self.writeRegister(REG_OP_MODE, LORA_STANDBY_MODE)	#LoRa Stdby mode to write in registers
				self.power=pow-2
				p=power()
				p.octet=self.readRegister(REG_PA_CONFIG)
				p.bits.OutputPower=self.power
				p.bits.PaSelect=1
				self.writeRegister(REG_PA_CONFIG,p.octet)
				self.writeRegister(REG_OP_MODE, st0)		# Getting back to previous status
			
			else:
				print('Power out of range')
				self.ko=1

		else:
			print("Not in LORA mode")



	def receive(self):
		if(self.modem==LORA):
			self.ft232h.setup(5, GPIO.OUT) #Pin d5 Rx (Purple)
			self.ft232h.setup(6, GPIO.OUT) #Pin d6 Tx (White)


			self.ft232h.output(5, GPIO.HIGH)
			self.ft232h.output(6, GPIO.LOW)


			self.writeRegister(REG_OP_MODE,LORA_STANDBY_MODE)
			self.writeRegister(REG_FIFO_ADDR_PTR,REG_FIFO_RX_BASE_ADDR)
			self.writeRegister(REG_OP_MODE,LORA_RX_MODE)
			self.payload=[]
			self.d=[]
			self.crcreceived=[]
			self.snrpacket=[]
			self.rssipacket=[]
			self.rssi=[]
			self.graph=collections.deque()
			self.packetNumber=0
		else:
			print("Not in LORA mode")

	def getpacketSnr(self):
		if(self.modem==LORA):
			a=self.readRegister(REG_PKT_SNR_VALUE)
			#print a
			return (self.a2_to_dec(a)/4)
		else:
			print("Not in LORA mode")

	def getpacketRssi(self):
		if(self.modem==LORA):
			return self.readRegister(REG_PKT_RSSI_VALUE)
		else:
			print("Not in LORA mode")

	def getRssi(self):
		if(self.modem==LORA):
			return self.readRegister(REG_RSSI_VALUE_LORA)
		else:
			print("Not in LORA mode")


	def validpacketsCnt(self):
		if(self.modem==LORA):
			a=ValidpacketsRX()
			a.bits.ValidPacketCntLsb=self.readRegister(REG_RX_PACKET_CNT_VALUE_LSB)
			print self.readRegister(0x17)
			a.bits.ValidPacketCntMsb=self.readRegister(REG_RX_PACKET_CNT_VALUE_MSB)
			return a.octet
		else:
			print("Not in LORA mode")

	def validheadersCnt(self):
		if(self.modem==LORA):
			a=ValidheadersRX()
			a.bits.ValidHeaderCntLsb=self.readRegister(REG_RX_HEADER_CNT_VALUE_LSB)
			a.bits.ValidHeaderCntMsb=self.readRegister(REG_RX_HEADER_CNT_VALUE_MSB)
			return a.octet
		else:
			print("Not in LORA mode")

			

	def getonePacket(self):
		if (self.readRegister(REG_OP_MODE)==LORA_RX_MODE):
			self.payload=[]
			fl=flags()
			fl.octet=self.readRegister(REG_IRQ_FLAGS)
			#print(hex(fl.octet))
			if (fl.bits.RxDone==1):
				#print(self.validheadersCnt())
				#print 'RxDone'
				if (fl.bits.ValidHeader==1):
					self.packetNumber=self.packetNumber+1
					#print (self.validheadersCnt())
					#print 'ValidHeader received from packet:',
					#print(self.packetNumber)
					hop=HopChannel()
					hop.octet=self.readRegister(REG_HOP_CHANNEL)
					if (hop.bits.CrcOnPayload==1):
						#print('Header indicates CRC on')
						if (fl.bits.PayloadCrcError==1):
							#print'PayloadCrcError from packet:',
							#print(self.packetNumber)
							self.crcreceived=['CRC_KO']
						else:
							self.crcreceived=['CRC_OK']
					else:
						self.crcreceived=['CRC_OFF']
					self.writeRegister(REG_OP_MODE,LORA_STANDBY_MODE)
					currentaddr=self.readRegister(REG_FIFO_RX_CURRENT_ADDR)
					#print hex(currentaddr)
					self.writeRegister(REG_FIFO_ADDR_PTR,currentaddr)
					for i in range(self.readRegister(REG_RX_NB_BYTES)):
						self.payload.append(hex(self.readRegister(REG_FIFO)))
					self.writeRegister(REG_FIFO_ADDR_PTR,self.readRegister(REG_FIFO_RX_BASE_ADDR))
					self.snrpacket=[self.getpacketSnr()]
					if (self.getpacketSnr>=0):
						self.rssipacket=[-139+self.getpacketRssi()]
					else:
						self.rssipacket=[-139+self.getpacketRssi()+0.25*self.getpacketSnr()]
					self.graph.append([self.packetNumber]+self.rssipacket+self.snrpacket) #Utilizar para grafico
					self.rssi=[-139+self.getRssi()]
					self.d=[self.packetNumber]+[self.readRegister(REG_RX_NB_BYTES)]+[self.payload]+self.crcreceived+self.snrpacket+self.rssipacket+self.rssi
					#print self.d
					self.queuerx.append(self.d)
					self.clearFlags()
					self.writeRegister(REG_OP_MODE,LORA_RX_MODE)

					#print(self.l)
					

		else:
			pass
			#print('Call function receive for putting modem in LORA_RX_MODE')






	def getPackets(self):
		if (self.readRegister(REG_OP_MODE)==LORA_RX_MODE):
			self.l=[]
			fl=flags()
			fl.octet=self.readRegister(REG_IRQ_FLAGS)
			#print(hex(fl.octet))
			if (fl.bits.RxDone==1):
				print 'RxDone'
				if (fl.bits.ValidHeader==1):
					self.packetNumber=self.packetNumber+1
					print 'ValidHeader received from packet:',
					print(self.packetNumber)
					hop=HopChannel()
					hop.octet=self.readRegister(REG_HOP_CHANNEL)
					if (hop.bits.CrcOnPayload==1):
						print('Header indicates CRC on')
						if (fl.bits.PayloadCrcError==1):
							print'PayloadCrcError from packet:',
							print(self.packetNumber)
							self.l=self.l+[1]
						else:
							self.l=self.l+[0]
					else:
						self.l=self.l+['CRC_OFF']
					self.writeRegister(REG_OP_MODE,LORA_STANDBY_MODE)
					currentaddr=self.readRegister(REG_FIFO_RX_CURRENT_ADDR)
					self.writeRegister(REG_FIFO_ADDR_PTR,currentaddr)
					for i in range(self.readRegister(REG_RX_NB_BYTES)):
						self.l=self.l+[self.readRegister(REG_FIFO)]
					self.queuerx.append(self.l)
					self.writeRegister(REG_FIFO_ADDR_PTR,self.readRegister(REG_FIFO_RX_BASE_ADDR))
					self.clearFlags()
					self.writeRegister(REG_OP_MODE,LORA_RX_MODE)
					self.l=[]
					

		else:
			print('Call function receive for putting modem in LORA_RX_MODE')

	def startRx(self):
		self.threadrx.start()

	def stopRx(self):
		self.threadrx.cancel()


 	#Transmission.........................


	def upload_buffer(self,packet):	#packet must be string
		if(self.modem==LORA):
			st0 = self.readRegister(REG_OP_MODE)	   #Save the previous status
			self.writeRegister(REG_OP_MODE,LORA_STANDBY_MODE)
			self.buffer=self.hex_to_byte(packet)
			print self.buffer
			self.writeRegister(REG_FIFO_TX_BASE_ADDR,0x00)
			self.writeRegister(REG_FIFO_ADDR_PTR,0x00)#REG_FIFO_TX_BASE_ADDR)
			
			self.ft232h.setup(5, GPIO.OUT) #Pin d5 Rx (Purple)
			self.ft232h.setup(6, GPIO.OUT) #Pin d6 Tx (White)


			self.ft232h.output(5, GPIO.LOW)
			self.ft232h.output(6, GPIO.HIGH)
			
			for i in range(len(self.buffer)):
				self.writeRegister(REG_FIFO,self.buffer[i])
				#print self.buffer[i]
			self.writeRegister(REG_PAYLOAD_LENGTH_LORA,len(self.buffer))
			self.writeRegister(REG_OP_MODE, st0)		# Getting back to previous status
		else:
			print("Not in LORA mode")
	

	def transmit(self):
		if(self.modem==LORA):
			if (len(self.buffer)>0):
				#self.ft232h.setup(5, GPIO.OUT) #Pin d5 Rx (Purple)
				#self.ft232h.setup(6, GPIO.OUT) #Pin d6 Tx (White)


				#self.ft232h.output(5, GPIO.LOW)
				#self.ft232h.output(6, GPIO.HIGH)

				self.writeRegister(REG_OP_MODE,LORA_TX_MODE)
				start_time=time.time()
				fl=flags()
				c=0
				while (time.time()-start_time<500):	#500 secons for transmission if more it will print an error
					fl.octet=self.readRegister(REG_IRQ_FLAGS)
					if (fl.bits.TxDone==1):
						print 'Transmission done'
						c=1
						break
				if (c==0):
					print 'Error during transmission'

				self.buffer=[]
			else:
				print('buffer is empty')
			self.clearFlags()
			self.ft232h.output(6, GPIO.LOW)
		else:
			print("Not in LORA mode")




	def setConfiguration(self,bw,codingrate,spredingfactor,preamblelenght,setchannel,setpower,crc):
		self.ON()
		self.setLORA()
		self.setBandwidth(bw)
		self.setCodingRate(codingrate)
		self.setSpreadingFactor(spredingfactor)
		self.setPreambleLength(preamblelenght)
		self.setHeaderON()
		#self.setLowDataRateOptimize()
		if crc=='ON':
			self.setCRC_ON()
		if crc=='OFF':
			self.setCRC_OFF()
		self.setChannel(setchannel)
		self.setPowerNum(setpower)

		# Checking configuration
		self.getMode()
		self.getHeader()
		self.getPreambleLength()
		self.getChannel()
		self.getPower()
		self.getCRC()

		#print hex(self.readRegister(REG_MODEM_CONFIG1))
		#print hex(self.readRegister(REG_MODEM_CONFIG2))
		#print hex(self.readRegister(0X31))
		#print hex(self.readRegister(0X37))
		print hex(self.readRegister(REG_FRF_MSB))
		print hex(self.readRegister(REG_FRF_MID))
		print hex(self.readRegister(REG_FRF_LSB))











	#Tools

	def a2_to_dec(self,value):	#convert from a2 complement to decimal with 8 bit
		i=2
		a2=format(value,'#010b')
		#print a2
		s=''
		if a2[2]=='1':
			while i<len(a2):
				if a2[i]=='0':
					s=s+'1'
				else:
					s=s+'0'
				i=i+1
			v=(int(s,2)+1)*-1
			return v
		else:
			return value


	def unicode_to_byte(self,unicode):	#returns a list of bytes encoded in unicode
		a=map(ord,unicode)
		print a
		l=[]
		for c in range(len(a)):
			a1=format(a[c],'#06x')
			v1=a1[2:4]
			v2=a1[4:]
			l=l+[int(v1,16)]+[int(v2,16)]
		print l
		return l


	def hex_to_byte(self,data):
		l=[]
		if len(data)%2==1:
			a='0'+data
		else:
			a=data
		for c in range(0,len(a)-1,2):
			l=l+[int(a[c:c+2],16)]
		#print l
		return l


